@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005, 2009 Apple Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -25,11 +25,32 @@
 #define _XLOCALE__TIME_H_
 
 __BEGIN_DECLS
+//Begin-Libc
+#ifndef LIBC_ALIAS_STRFTIME_L
+//End-Libc
 size_t	 strftime_l(char * __restrict, size_t, const char * __restrict,
 		const struct tm * __restrict, locale_t)
-		__DARWIN_ALIAS(strftime_l);
+		__DARWIN_ALIAS(strftime_l) __strftimelike(3);
+//Begin-Libc
+#else /* LIBC_ALIAS_STRFTIME_L */
+size_t	 strftime_l(char * __restrict, size_t, const char * __restrict,
+		const struct tm * __restrict, locale_t)
+		LIBC_ALIAS(strftime_l) __strftimelike(3);
+#endif /* !LIBC_ALIAS_STRFTIME_L */
+//End-Libc
+//Begin-Libc
+#ifndef LIBC_ALIAS_STRPTIME_L
+//End-Libc
 char	*strptime_l(const char * __restrict, const char * __restrict,
-		struct tm * __restrict, locale_t);
+		struct tm * __restrict, locale_t) 
+		__DARWIN_ALIAS(strptime_l) __strftimelike(2);
+//Begin-Libc
+#else /* LIBC_ALIAS_STRPTIME_L */
+char	*strptime_l(const char * __restrict, const char * __restrict,
+		struct tm * __restrict, locale_t)
+		LIBC_ALIAS(strptime_l) __strftimelike(2);
+#endif /* !LIBC_ALIAS_STRPTIME_L */
+//End-Libc
 __END_DECLS
 
 #endif /* _XLOCALE__TIME_H_ */
