@@ -25,7 +25,9 @@
  */
 
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: src/lib/libc/locale/mbstowcs.c,v 1.11 2004/07/21 10:54:57 tjr Exp $");
+__FBSDID("$FreeBSD: src/lib/libc/locale/mbstowcs.c,v 1.12 2009/01/15 18:53:52 rdivacky Exp $");
+
+#include "xlocale_private.h"
 
 #include <limits.h>
 #include <stdlib.h>
@@ -33,11 +35,21 @@ __FBSDID("$FreeBSD: src/lib/libc/locale/mbstowcs.c,v 1.11 2004/07/21 10:54:57 tj
 #include "mblocal.h"
 
 size_t
-mbstowcs(wchar_t * __restrict pwcs, const char * __restrict s, size_t n)
+mbstowcs_l(wchar_t * __restrict pwcs, const char * __restrict s, size_t n,
+    locale_t loc)
 {
 	static const mbstate_t initial;
 	mbstate_t mbs;
+	const char *sp;
 
+	NORMALIZE_LOCALE(loc);
 	mbs = initial;
-	return (__mbsnrtowcs(pwcs, &s, SIZE_T_MAX, n, &mbs));
+	sp = s;
+	return (loc->__lc_ctype->__mbsnrtowcs(pwcs, &sp, SIZE_T_MAX, n, &mbs, loc));
+}
+
+size_t
+mbstowcs(wchar_t * __restrict pwcs, const char * __restrict s, size_t n)
+{
+	return mbstowcs_l(pwcs, s, n, __current_locale());
 }
